{**********************************************************************
                PilotLogic Software House

 Package ds_Pas2JavaScript_IDE
 This file is part of CodeTyphon Studio (https://www.pilotlogic.com/)
***********************************************************************}

unit allpas2jsidereg;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,
  // LCL
  Forms, Controls,
  // BaseUtils
  LazLoggerBase,
  // IdeIntf
  ProjectIntf, ComponentEditors, PropEdits, LazIDEIntf, IDEOptionsIntf, IDEOptEditorIntf,
  FormEditingIntf, NewItemIntf,
  IDEWindowIntf, IDEMsgIntf, IDEExternToolIntf,
  // Pas2js
  PJSDsgnOptions, PJSDsgnOptsFrame,
  pjsdsgnregister, idehtml2class,

  WebCtrls,
  Pas2JS_IDE_Descriptor;


procedure Register;

implementation

{$R allpas2jsidereg.res}

uses
  dialogs,
  frmPas2jsSWServersManager,
  frmpas2jsnodejsprojectoptions,
  frmpas2jsbrowserprojectoptions,
  pjsprojectoptions,
  idehtmltools,
  frmhtmltoform,
  regpas2jscomponents,
  regpas2jsvscode, regpas2jsatom,
  fpjson, pjscontroller, srceditorintf, strpas2jsdesign, IDECommands, ToolbarIntf, MenuIntf;

Const
  sPas2JSWebserverName = 'Pas2JSWebservers';


type
  TPas2JSWidgetsRequirements = class(TComponentRequirements)
  public
    procedure RequiredUnits(aUnits: TStrings); override;
    procedure RequiredPkgs(aPkgs: TStrings); override;
  end;


{ TPas2JSWidgetsRequirements }

procedure TPas2JSWidgetsRequirements.RequiredUnits(aUnits: TStrings);
begin
  if ComponentClass.ClassType = TWStringGrid then begin
    aUnits.Clear;
    aUnits.Add('Grids');
  end;
end;

procedure TPas2JSWidgetsRequirements.RequiredPkgs(aPkgs: TStrings);
begin
  aPkgs.Clear;
  aPkgs.Add('js_WebWidgetsGUI');
end;

//--------------------------------------------------------------

procedure Register;
Const
  Pas2JSCategory= 'Pas2JS';
Var
  ViewCategory : TIDECommandCategory;
  IDECommand : TIDECommand;
  SrvWorker: TProjectPas2JSServiceWorker;
  PWA: TProjectPas2JSProgressiveWebApp;
  MenuHandler:TPas2JSHandler;
begin

  //... From pjsdsgnregister ...........................................
  MenuHandler:=TPas2JSHandler.Create;
  if Assigned(MenuHandler) then; // Silence compiler warning
  PJSOptions:=TPas2jsOptions.Create;
  PJSOptions.Load;
  TPJSController.Instance.Hook;

  // Create New Category For ALL Pas2JS items
  RegisterNewItemCategory(TNewIDEItemCategory.Create(Pas2JSCategory),'ct_nitem_folder_pas2js');

  //=== Register new-File items =================================

    //... From Pas2JS_IDE_Descriptor ............
  VPas2JSWForm := TPas2JSWForm.Create;
  VPas2JSWFrame := TPas2JSWFrame.Create;
  VPas2JSWDataModule := TPas2JSWDataModule.Create;

  RegisterProjectFileDescriptor(VPas2JSWForm,Pas2JSCategory,'ct_nitem_formweb');
  RegisterProjectFileDescriptor(VPas2JSWFrame,Pas2JSCategory,'ct_nitem_frameweb');
  RegisterProjectFileDescriptor(VPas2JSWDataModule,Pas2JSCategory,'ct_nitem_datamoduleweb');

    //... From pjsdsgnregister ................
  Pas2JSHTMLClassDef:=TPas2JSHTMLClassDef.Create;
  RegisterProjectFileDescriptor(Pas2JSHTMLClassDef,Pas2JSCategory);
  Pas2JSDTSToPasUnitDef:=TPas2JSDTSToPasUnitDef.Create;
  RegisterProjectFileDescriptor(Pas2JSDTSToPasUnitDef,Pas2JSCategory);

   //... From regpas2jscomponents ................
  FragmentDescr:=TFileDescHTMLFragment.Create;
  RegisterProjectFileDescriptor(FragmentDescr,Pas2JSCategory);

  //=== Register new-Project items =============================

    //... From Pas2JS_IDE_Descriptor ...........
  VPas2JSProject := TPas2JSProject.Create;
  RegisterProjectDescriptor(VPas2JSProject,Pas2JSCategory);

    //... From pjsdsgnregister ................    
  RegisterProjectDescriptor(TProjectPas2JSWebApp.Create, Pas2JSCategory);

  PWA:=TProjectPas2JSProgressiveWebApp.Create;
  RegisterProjectDescriptor(PWA, Pas2JSCategory,'ct_nitem_multiproject');

  SrvWorker:=TProjectPas2JSServiceWorker.Create;
  RegisterProjectDescriptor(SrvWorker,Pas2JSCategory);
  PWA.ServiceWorker:=SrvWorker;

  RegisterProjectDescriptor(TProjectPas2JSModuleApp.Create, Pas2JSCategory);

  RegisterProjectDescriptor(TProjectPas2JSNodeJSApp.Create, Pas2JSCategory);

  RegisterProjectDescriptor(TProjectPas2JSElectronWebApp.Create, Pas2JSCategory,'ct_nitem_multiproject');

    //... From regPas2jsAtom .................
  AtomProjDesc:=TAtomPackageProjectDescriptor.Create();
  RegisterProjectDescriptor(AtomProjDesc,Pas2JSCategory);

    //... From regpas2jsvscode ...............
  VSCodeProjDesc:=TVSCodeExtensionProjectDescriptor.Create();
  RegisterProjectDescriptor(VSCodeProjDesc,Pas2JSCategory);

  //=========================================================================================
  // add IDE options frame
  PJSOptionsFrameID:=RegisterIDEOptionsEditor(GroupEnvironment,TPas2jsOptionsFrame,
                                              PJSOptionsFrameID)^.Index;

  // Add project options frame
  RegisterIDEOptionsEditor(GroupProject,TPas2JSProjectOptionsFrame, Pas2JSOptionsIndex);

  // pop menu items
  SrcMnuItem:=RegisterIDEMenuCommand(SrcEditMenuSectionFirstStatic,
     'HTMLFormClassRefresh', pjsRefreshClassFromHTML,@MenuHandler.OnRefreshHTMLFormContext);
  SourceEditorMenuRoot.AddHandlerOnShow(@MenuHandler.OnSrcEditPopup);
  PrjMnuItem:=RegisterIDEMenuCommand(ProjInspMenuSectionFiles,
      'PrjHTMLFormClassRefresh',pjsRefreshClassFromHTML,@MenuHandler.OnRefreshProjHTMLFormContext);
  PrjMnuItemAll:=RegisterIDEMenuCommand(ProjInspMenuSectionFiles,
      'PrjHTMLFormClassRefreshAll',pjsRefreshAllClassesFromHTML,@MenuHandler.OnRefreshProjHTMLFormAllContext);
  ProjectInspectorItemsMenuRoot.AddHandlerOnShow(@MenuHandler.OnPrjInspPopup);

  //=== SWServers ===========================================================================
  ViewCategory:=IDECommandList.CreateCategory(nil, 'SWServers Category', 'SWServers Category', IDECmdScopeSrcEditOnly);
  if ViewCategory<>nil then
    begin

     // Windows - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     PasJSSWSDlgWindowCreator:=IDEWindowCreators.Add(cnPasJSSWSDlgWindowName,
            @CreatePasJSSWSDlgWindow,nil,
           '20%','20%','+550','+425');

     // Windows - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     ViewPasJSSWSDlgWindowCommand:=RegisterIDECommand(ViewCategory, 'SimpleWebServerManager',
           'SWServers Manager', CleanIDEShortCut, CleanIDEShortCut, nil, @ShowPasJSSWSDlgWindow);
     RegisterIDEMenuCommand(itmViewMainWindows, 'ViewSimpleWebServerManagerIDE',
           'SWServers Manager', nil, nil, ViewPasJSSWSDlgWindowCommand,'ctsimplewebserver');
   end;

  //=== From Pas2JS_IDE_Descriptor ===========================================================

  RegisterComponentRequirements([
                                 TWPopupMenu,
                                 TWComboBox,
                                 TWListBox,
                                 TWEdit,
                                 TWMemo,
                                 TWButton,
                                 TWCheckbox,
                                 TWRadioButton,
                                 TWLabel,
                                 TWImage,
                                 TWPanel,
                                 TWTimer,
                                 TWPageControl,
                                 TWFloatEdit,
                                 TWIntegerEdit,
                                 TWDateEditBox,
                                 TWTimeEditBox,
                                 TWFileButton,
                                 TWStringGrid,
                                 TWSplitter,
                                 TWDataGrid,
                                 TWPagination,
                                 TWWebSocketClient
                                 ], TPas2JSWidgetsRequirements);

  //... From WebCtrls ................................................................

  FormEditingHook.RegisterDesignerBaseClass(TWForm);
  FormEditingHook.RegisterDesignerBaseClass(TWFrame);
  FormEditingHook.RegisterDesignerBaseClass(TWDataModule);

  RegisterComponents('jsGUI Base', [
                                TWPopupMenu,
                                TWComboBox,
                                TWListBox,
                                TWEdit,
                                TWMemo,
                                TWButton,
                                TWCheckbox,
                                TWRadioButton,
                                TWLabel,
                                TWImage,
                                TWPanel,
                                TWTimer,
                                TWPageControl,
                                TWFloatEdit,
                                TWIntegerEdit,
                                TWDateEditBox,
                                TWTimeEditBox,
                                TWFileButton,
                                TWStringGrid,
                                TWSplitter,
                                TWDataGrid,
                                TWPagination,
                                TWWebSocketClient
                                ]);

end;


end.


