unit PJSController;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, process,
  ctUtils,                                         //=== ct9999 =====
  // BaseUtils
  LazLoggerBase, LazUtilities, FileUtil, LazFileUtils, AvgLvlTree,
  // LCL
  Forms, Controls, LazHelpIntf,
  // IdeIntf
  MacroIntf, MacroDefIntf, ProjectIntf, CompOptsIntf, IDEExternToolIntf,
  LazIDEIntf, multiprojectintf,                    //=== ct9999 =====
  // pas2js
  StrPas2JSDesign, PJSDsgnOptions, CodeToolManager, //=== ct9999 =====
  CodeCache;

Type

//=== ct9999 ============================================
  { TServerInstance }

  TServerInstance = Class(TCollectionItem)
  private
    FlastProject: String;
    FPort: Word;
    FProcess: TProcess;
    FRunError: String;
    FServerName: String;
    FString: String;
    function GetRunning: Boolean;
  Protected
    Property Process : TProcess Read FProcess;
  Public
    Destructor Destroy; override;
    Procedure StartServer;
    Procedure StopServer;
    Property Port : Word Read FPort Write FPort;
    Property BaseDir : String Read FString Write FString;
    Property ServerName : String Read FServerName Write FServerName;
    Property Running : Boolean Read GetRunning;
    Property RunError : String Read FRunError;
    Property LastProject : String Read FlastProject Write Flastproject;
  end;

  { TServerInstanceList }

  TServerInstanceList = Class(TCollection)
  private
    function GetInstance(AIndex : Integer): TServerInstance;
  Public
    Function IndexOfPort(APort: Word) : integer;
    Function FindByPort(Aindex : Integer) : TServerInstance;
    Function AddInstance(aPort : Word; Const ABaseURL, aServerName : String) : TServerInstance;
    Property Instances [AIndex : Integer] : TServerInstance Read GetInstance; default;
  end;
//============================================================  
  
  { TPJSController }

  TPJSController = Class
  Private
    FMacroPas2js: TTransferMacro;
    FOnRefresh: TNotifyEvent;
    //=== ct9999 ===================
    FServerInstances: TServerInstanceList;
    function GetPas2JSWebServerPath(const s: string; const {%H-}Data: PtrInt; var Abort: boolean): string;
    function GetPas2JSWebServerPort(const s: string; const {%H-}Data: PtrInt; var Abort: boolean): string;
    //==============================
    function GetPas2JSPath(const s: string; const {%H-}Data: PtrInt; var Abort: boolean): string;
    function GetPas2JSBrowser(const s: string; const {%H-}Data: PtrInt; var Abort: boolean): string;
    function GetPas2JSNodeJS(const s: string; const {%H-}Data: PtrInt; var Abort: boolean): string;
    function GetPas2JSElectron(const s: string; const {%H-}Data: PtrInt; var Abort: boolean): string;
    function GetPas2jsProjectURL(const s: string; const {%H-}Data: PtrInt; var Abort: boolean): string;
    function OnProjectBuilding(Sender: TObject): TModalResult;
    function OnMultiProjectRunTyphonbuild({%H-}Target: TMPCompileElement; Tool: TAbstractExternalTool): boolean; //=== ct9999 ====
    function OnRunDebugInit(Sender: TObject; var Handled: boolean
      ): TModalResult;
    function OnRunWithoutDebugInit(Sender: TObject; var Handled: boolean): TModalResult;
    function RunProject(Sender: TObject; WithDebug: boolean; var Handled: boolean): TModalResult;
    function RunBrowserProject(aProject: TTyphonProject; WithDebug: boolean; var Handled: boolean): TModalResult;
    function RunNonBrowserProject(aProject: TTyphonProject; WithDebug: boolean; var Handled: boolean): TModalResult;
    function SaveHTMLFileToTestDir(aProject: TTyphonProject): boolean;
    class function GetProjectHTMLLegacyFilename(aProject: TTyphonProject): string;
  Public
    Constructor Create;
    Destructor Destroy; override;
    Class Procedure DoneInstance;
    Class Function Instance: TPJSController;
    Procedure Hook; virtual;
    Procedure UnHook; virtual;   
    Procedure StoreMacros; virtual;
    //=== ct9999 =============================
    Procedure RefreshView;
    Property ServerInstances : TServerInstanceList Read FServerInstances;
    //========================================
    // Determine project HTML file from custom data
    class function GetProjectHTMLFile(aProject: TTyphonProject): TTyphonProjectFile;
    class function GetProjectHTMLFilename(aProject: TTyphonProject): string;
    // Get filename to show in browser when running
    function GetHTMLFilename(aProject: TTyphonProject; UseTestDir: boolean): string; virtual;
    function GetWebDir(aProject: TTyphonProject): string; virtual; // disk directory for webserver
    function GetProjectLocation(aProject: TTyphonProject): string; virtual;
    function GetProjectURL(aProject: TTyphonProject): string; virtual;
    Property OnRefresh : TNotifyEvent Read FOnRefresh Write FonRefresh;
    property MacroPas2js: TTransferMacro read FMacroPas2js;
  end;

Const
  // Custom settings in .ctpr
  PJSProject = 'Pas2JSProject'; // Project is pas2js project
  PJSProjectWebBrowser =  'PasJSWebBrowserProject'; // Web browser project
  PJSProjectHTMLFile = 'PasJSHTMLFile' deprecated 'use TPJSController.GetProjectHTMLFilename'; // No longer used
  PJSIsProjectHTMLFile = 'PasJSIsProjectHTMLFile';
  PJSProjectMaintainHTML = 'MaintainHTML';
  PJSProjectUseBrowserConsole = 'BrowserConsole';
  PJSProjectRunAtReady = 'RunAtReady';
  PJSProjectLocation = 'PasJSLocation';
  PJSProjectPort = 'PasJSPort';
  PJSProjectURL = 'PasJSURL';
  PJSProjectHTMLBaseDir = 'HTMLDir';
  PJSUseHTTPServer = 'PasJSUseHTTPServer';   //=== ct9999 ===================


implementation

Var
  ctrl : TPJSController;

{ TPJSController }

class procedure TPJSController.DoneInstance;
begin
  FreeAndNil(Ctrl)
end;

class function TPJSController.Instance: TPJSController;
begin
  if Ctrl=Nil then
    Ctrl:=TPJSController.Create;
  Result:=Ctrl;
end;

function TPJSController.GetPas2JSPath(const s: string; const Data: PtrInt;
  var Abort: boolean): string;
begin
  Abort:=False;
  if (s<>'') and (ConsoleVerbosity>=0) then
    debugln(['Hint: (IDE) [TPJSController.GetPas2JSPath] ignoring macro Pas2JS parameter "',s,'"']);
  Result:=PJSOptions.GetParsedCompilerFilename;
  if Result='' then
    Result:='pas2js'; // always return something to get nicer error messages
end;

//=== ct9999 =========================================================
function TPJSController.GetPas2JSBrowser(const s: string; const Data: PtrInt; var Abort: boolean): string;

begin
  Abort:=False;
  if (s<>'') and (ConsoleVerbosity>=0) then
    debugln(['Hint: (IDE) [TPJSController.GetPas2JSBrowser] ignoring macro Pas2JSBrowser parameter "',s,'"']);
  Result:=PJSOptions.GetParsedBrowserFilename;

  if Result='' then
   {$if defined(MSWINDOWS)}

    // Try FireFox 64
    if FileExists('C:\Program Files\Mozilla Firefox\firefox.exe') then
    Result:='C:\Program Files\Mozilla Firefox\firefox.exe';

    // Try FireFox 32
    if FileExists('C:\Program Files (x86)\Mozilla Firefox\firefox.exe') then
    Result:='C:\Program Files (x86)\Mozilla Firefox\firefox.exe';

    // Try Chrome
    if Result='' then
    if FileExists('C:\Program Files (x86)\Google\Chrome\Application\chrome.exe') then
    Result:='C:\Program Files (x86)\Google\Chrome\Application\chrome.exe';

    // Try MS Edge
    if Result='' then
    if FileExists('C:\Program Files (x86)\Microsoft\Edge\Application\msedge.exe') then
    Result:='C:\Program Files (x86)\Microsoft\Edge\Application\msedge.exe';

    if Result='' then
    Result:='firefox'; // always return something to get nicer error messages

   {$elseif defined(DARWIN)}
    Result:='/usr/bin/open -a Safari';
   {$elseif defined(DragonFly)}
    Result:='/usr/local/bin/firefox';
   {$ELSE}
    Result:='firefox'; // always return something to get nicer error messages
   {$ENDIF}
end;
//===============================================================================

function TPJSController.GetPas2JSNodeJS(const s: string; const Data: PtrInt; var Abort: boolean): string;

begin
  Abort:=False;
  if (s<>'') and (ConsoleVerbosity>=0) then
    debugln(['Hint: (IDE) [TPJSController.GetPas2JSNodeJS] ignoring macro Pas2JSNodeJS parameter "',s,'"']);
  Result:=PJSOptions.GetParsedNodeJSFilename;
  if Result='' then
    Result:='nodejs'+GetExeExt; // always return something to get nicer error messages
end;

function TPJSController.GetPas2JSElectron(const s: string; const Data: PtrInt;
  var Abort: boolean): string;
begin
  Abort:=False;
  if (s<>'') and (ConsoleVerbosity>=0) then
    debugln(['Hint: (IDE) [TPJSController.GetPas2JSElectron] ignoring macro Pas2JSElectron parameter "',s,'"']);
  Result:=PJSOptions.GetParsedElectronExe;
  if Result='' then
    Result:='electron'+GetExeExt; // always return something to get nicer error messages
end;

function TPJSController.GetPas2jsProjectURL(const s: string; const Data: PtrInt; var Abort: boolean): string;
Var
  aProject: TTyphonProject;
begin
  if (s<>'') and (ConsoleVerbosity>=0) then
    debugln(['Hint: (IDE) [TPJSController.GetPas2jsProjectURL] ignoring macro Pas2JSProjectURL parameter "',s,'"']);

  aProject:=TyphonIDE.ActiveProject;
  if ConsoleVerbosity>0 then
    DebugLN(['Hint: (IDE) [TPJSController.GetPas2jsProjectURL] TyphonIDE.ActiveProject.CustomData[PJSProjectWebBrowser]: ',aProject.CustomData[PJSProjectWebBrowser]]);
  Abort:=aProject.CustomData[PJSProjectWebBrowser]<>'1';
  if Abort then
    exit;
  if ConsoleVerbosity>0 then
    DebugLN(['Hint: (IDE) [TPJSController.GetPas2jsProjectURL] TyphonIDE.ActiveProject.CustomData[PJSProjectURL]: ',aProject.CustomData[PJSProjectURL]]);
  Result:=GetProjectURL(aProject);
  Abort:=(Result='');
  if ConsoleVerbosity>0 then
    DebugLN(['Hint: (IDE) [TPJSController.GetPas2jsProjectURL] Result="',Result,'"']);
end;

function TPJSController.OnProjectBuilding(Sender: TObject): TModalResult;
var
  aProject: TTyphonProject;
begin
  Result:=mrOk;
  aProject:=TyphonIDE.ActiveProject;
  if aProject=nil then exit;
  StoreMacros;
  if aProject.IsVirtual then
    begin
    if not SaveHTMLFileToTestDir(aProject) then
      exit(mrCancel);
    end;
end;

//=== ct9999 =========================
function TPJSController.OnMultiProjectRunTyphonbuild(Target: TMPCompileElement; Tool: TAbstractExternalTool): boolean;
var
  Pas2jsFilename: String;
begin
  Result:=true;
  Pas2jsFilename:=PJSOptions.GetParsedCompilerFilename;
  if Pas2jsFilename<>'' then
    Tool.EnvironmentOverrides.Values['PAS2JS']:=Pas2jsFilename;
end;
//====================================

function TPJSController.OnRunDebugInit(Sender: TObject; var Handled: boolean
  ): TModalResult;
begin
  Result:=RunProject(Sender,true,Handled);
end;

function TPJSController.OnRunWithoutDebugInit(Sender: TObject; var Handled: boolean): TModalResult;
begin
  Result:=RunProject(Sender,false,Handled);
end;

class function TPJSController.GetProjectHTMLFile(aProject: TTyphonProject
  ): TTyphonProjectFile;
var
  HTMLFilename: String;
  i: Integer;
begin
  for i:=0 to aProject.FileCount-1 do
    begin
    Result:=aProject.Files[i];
    if Result.IsPartOfProject
        and (Result.CustomData[PJSIsProjectHTMLFile]='1') then
      exit;
    end;
  Result:=nil;

  HTMLFilename:=GetProjectHTMLLegacyFilename(aProject);
  if HTMLFilename<>'' then
    Result:=aProject.FindFile(HTMLFilename,[pfsfOnlyProjectFiles]);
end;

class function TPJSController.GetProjectHTMLFilename(aProject: TTyphonProject): string;

Var
  aFile: TTyphonProjectFile;

begin
  aFile:=GetProjectHTMLFile(aProject);
  if aFile<>nil then
    Result:=aFile.GetFullFilename
  else
    Result:='';
end;

function TPJSController.GetHTMLFilename(aProject: TTyphonProject; UseTestDir: boolean): string;
begin
  Result:=GetProjectHTMLFileName(aProject);
  if Result='' then exit;
  if FilenameIsAbsolute(Result) then exit;

  if aProject.IsVirtual then
    begin
    if UseTestDir then
      Result:=AppendPathDelim(TyphonIDE.GetTestBuildDirectory)+Result;
    end
  else
    Result:=TrimFilename(AppendPathDelim(aProject.Directory)+Result);
end;

function TPJSController.GetWebDir(aProject: TTyphonProject): string;
var
  HTMLFilename: String;
begin
  Result:='';

  HTMLFilename:=GetHTMLFilename(aProject,true);
  if HTMLFilename<>'' then
    Result:=ExtractFilePath(HTMLFilename);

  if Result='' then
    if aProject.IsVirtual then
      Result:=TyphonIDE.GetTestBuildDirectory
    else
      Result:=ExtractFilePath(aProject.ProjectInfoFile);
end;

function TPJSController.GetProjectLocation(aProject: TTyphonProject): string;
begin
  Result:=aProject.CustomData[PJSProjectLocation];
  IDEMacros.SubstituteMacros(Result);
end;

function TPJSController.GetProjectURL(aProject: TTyphonProject): string;
Var
  HTMLFilename, WebDir: String;
  Port: LongInt;
  //=== ct9999 =========================
  xServB:Boolean;
  xPortS:String;
  xSS:String;
  //====================================
begin
  if aProject=nil then
    exit('');
  Result:=aProject.CustomData[PJSProjectURL];
  if Result<>'' then
    begin
    IDEMacros.SubstituteMacros(Result);
    exit;
    end;

  if Result='' then
    begin
    Port:=StrToIntDef(aProject.CustomData[PJSProjectPort],-1);
    HTMLFilename:=GetHTMLFilename(aProject,true);
    if HTMLFilename='' then
      begin
      if Port<=0 then
        HTMLFilename:=ChangeFileExt(aProject.ProjectInfoFile,'.html')
      else
        HTMLFilename:=ExtractFileNameOnly(aProject.ProjectInfoFile)+'.html';
      end
    else if Port>0 then
      begin
      WebDir:=GetWebDir(aProject);
      HTMLFilename:=CreateRelativePath(HTMLFilename,WebDir);
      end;
    HTMLFilename:=FilenameToURLPath(HTMLFilename);
                                                                 
  //=== ct9999 ================================         
    xServB:=(TyphonIDE.ActiveProject.CustomData[PJSUseHTTPServer]='1');
    xPortS:=TyphonIDE.ActiveProject.CustomData[PJSProjectPort];

    if xServB and (xPortS<>'') and (xPortS<>'0') then
      begin
         Result:=Format('http://localhost:%s/%s',[xPortS,HTMLFilename]);
      end else
      begin
        xSS:=HTMLFilename;

        if FileExists(xss)=False then
          begin
            xss:=ExtractFilePath(TyphonIDE.ActiveProject.ProjectInfoFile)+HTMLFilename;  // .ctpr
          end;

        if FileExists(xss)=False then
          begin
            xss:=ExtractFilePath(TyphonIDE.ActiveProject.ProjectInfoFile)+WebDir+HTMLFilename; // .ctpr + WebDir
          end;

        {$IFDEF WINDOWS}
           Result:=Format('file:///%s',[xSS]);
        {$ELSE}
           Result:=Format('file://%s',[xSS]);
        {$ENDIF}
          end;

      end;
  //===========================================
end;

function TPJSController.RunProject(Sender: TObject; WithDebug: boolean;
  var Handled: boolean): TModalResult;
var
  aProject: TTyphonProject;
  IsWebProject, IsPSProject: Boolean;
begin
  Result:=mrOk;
  if Sender=nil then ;

  aProject:=TyphonIDE.ActiveProject;
  if aProject=nil then exit;

  IsWebProject:=aProject.CustomData[PJSProjectWebBrowser]='1';
  if IsWebProject then
    begin
    Result:=RunBrowserProject(aProject,WithDebug,Handled);
    exit;
    end;

  IsPSProject:=aProject.CustomData[PJSProject]='1';
  if IsPSProject then
    begin
    Result:=RunNonBrowserProject(aProject,WithDebug,Handled);
    exit;
    end;
end;

//=== ct9999 ===========================================================
function TPJSController.RunBrowserProject(aProject: TTyphonProject;
  WithDebug: boolean; var Handled: boolean): TModalResult;
var
  IsWebProject: Boolean;

  UseHTTPServer: Boolean;
  ServerPort : Word;
  WebProject : Boolean;
  BaseDir : String;
  aInstance : TServerInstance;
begin
  Result:=mrOk;

  aProject:=TyphonIDE.ActiveProject;
  if aProject=nil then Exit(mrCancel);

  IsWebProject:=aProject.CustomData[PJSProjectWebBrowser]='1';
  if not IsWebProject then
    Exit(mrCancel);

  //....................................................
  With TyphonIDE.ActiveProject do
    begin
    if ConsoleVerbosity>=0 then
      begin
      DebugLn(['WebProject=',CustomData[PJSProjectWebBrowser]]);
      DebugLn(['ServerPort=',CustomData[PJSProjectPort]]);
      DebugLn(['BaseDir=',ProjectInfoFile]);
      end;
    WebProject:=CustomData[PJSProjectWebBrowser]='1';
    ServerPort:=StrToIntDef(CustomData[PJSProjectPort],0);
    BaseDir:=ExtractFilePath(ProjectInfoFile);
    UseHTTPServer:=CustomData[PJSUseHTTPServer]='1';
    end;

  // Exit if we don't need to do anything
  if Not (WebProject and UseHTTPServer) then  // Start Only if UseHTTPServer
    Exit(mrCancel);

  aInstance:=ServerInstances.FindByPort(ServerPort);

  If Ainstance<>Nil then
    begin
      DebugLN('[TPJSController.MaybeStartServer] Have instance running on port '+IntToStr(ServerPort));

      if Not SameFileName(BaseDir,aInstance.BaseDir) then
      begin
        DebugLN('[TPJSController.MaybeStartServer] Instance on port '+IntToStr(ServerPort)+
                ' serves different directory: '+aInstance.BaseDir);

        If aInstance.Running then
           aInstance.StopServer;

        ServerInstances.Delete(aInstance.Index);
        aInstance:=ServerInstances.AddInstance(ServerPort,BaseDir,PJSOptions.GetParsedWebServerFilename);
      end else
      begin
        Exit(mrCancel);
      end;

    end else
    begin
      aInstance:=ServerInstances.AddInstance(ServerPort,BaseDir,PJSOptions.GetParsedWebServerFilename);
    end;

  aInstance.LastProject:=TyphonIDE.ActiveProject.ProjectInfoFile;
  aInstance.StartServer;
  Handled:=False;
  //....................................................

end;
//=========================================================================

function TPJSController.RunNonBrowserProject(aProject: TTyphonProject;
  WithDebug: boolean; var Handled: boolean): TModalResult;
begin
  Result:=mrOk;

  if not WithDebug then
    exit; // compile normally and run the run-parameters

  if aProject=nil then ;

  // for now: redirect to run without debug
  Handled:=true;
  Result:=TyphonIDE.DoRunProjectWithoutDebug;
end;

function TPJSController.SaveHTMLFileToTestDir(aProject: TTyphonProject): boolean;
var
  HTMLFilename, FullHTMLFilename: String;
  HTMLFile: TTyphonProjectFile;
  Code: TCodeBuffer;
begin
  // if project has a pas2js html filename, save it to the test directory
  Result:=false;
  HTMLFilename:=aProject.CustomData.Values[PJSProjectHTMLFile];
  if (HTMLFilename='') then
    exit(true);
  if FilenameIsAbsolute(HTMLFilename) then
    begin
    debugln(['Warning: TPJSController.SaveHTMLFileToTestDir html file is absolute: "',HTMLFilename,'"']);
    exit(true);
    end;
  HTMLFile:=aProject.FindFile(HTMLFilename,[pfsfOnlyProjectFiles]);
  if HTMLFile=nil then
    begin
    debugln(['Warning: TPJSController.SaveHTMLFileToTestDir invalid aProject.CustomData.Values[',PJSProjectHTMLFile,']']);
    exit;
    end;
  HTMLFilename:=HTMLFile.Filename;

  Code:=CodeToolBoss.FindFile(HTMLFilename);
  if Code=nil then
    begin
    debugln(['Error: TPJSController.SaveHTMLFileToTestDir missing codebuffer "',HTMLFilename,'"']);
    exit;
    end;

  FullHTMLFilename:=AppendPathDelim(TyphonIDE.GetTestBuildDirectory)+HTMLFilename;
  if not Code.SaveToFile(FullHTMLFilename) then
    begin
    debugln(['Error: TPJSController.SaveHTMLFileToTestDir write error: ',Code.LastError,' File="',FullHTMLFilename,'"']);
    exit;
    end;

  Result:=true;
end;

class function TPJSController.GetProjectHTMLLegacyFilename(aProject: TTyphonProject
  ): string;
begin
  Result:=aProject.CustomData.Values[PJSProjectHTMLFile{%H-}];
  if Result='' then exit;
  DoDirSeparators(Result);
  if (not aProject.IsVirtual) and (not FilenameIsAbsolute(Result)) then
    Result:=TrimFilename(AppendPathDelim(aProject.Directory)+Result);
end;

constructor TPJSController.Create;
begin
  FServerInstances:=TServerInstanceList.Create(TServerInstance); //=== ct9999 =====
end;

destructor TPJSController.Destroy;
begin
  Unhook;
  FreeAndNil(FServerInstances);   //=== ct9999 =====
  inherited Destroy;
end;

procedure TPJSController.Hook;
begin
  FMacroPas2js:=IDEMacros.Add('Pas2JS', '', pjsdPas2JSExecutable, @GetPas2JSPath, [tmfTyphonbuild]);
  
//=== ct9999 ==========================================    
  IDEMacros.Add(TTransferMacro.Create('Pas2JSWebServer', '', pjsdPas2JSWebServerExe, @
    GetPas2JSWebServerPath, []));
  IDEMacros.Add(TTransferMacro.Create('Pas2JSWebServerPort', '', pjsdPas2JSWebServerPort, @
    GetPas2JSWebServerPort, []));
//=====================================================    
  IDEMacros.Add(TTransferMacro.Create('Pas2JSBrowser', '',
    pjsdPas2JSSelectedBrowserExecutable, @GetPas2JSBrowser, []));
  IDEMacros.Add(TTransferMacro.Create('Pas2JSNodeJS', '',
    pjsdPas2JSSelectedNodeJSExcutable, @GetPas2JSNodeJS, []));
  IDEMacros.Add(TTransferMacro.Create('Pas2JSElectron', '',
    pjsdPas2JSSelectedElectronExcutable, @GetPas2JSElectron, []));
  IDEMacros.Add(TTransferMacro.Create('Pas2JSProjectURL', '',
    pjsdPas2JSCurrentProjectURL, @GetPas2jsProjectURL, []));

  TyphonIDE.AddHandlerOnProjectBuilding(@OnProjectBuilding);
  TyphonIDE.AddHandlerOnRunDebugInit(@OnRunDebugInit);
  TyphonIDE.AddHandlerOnRunWithoutDebugInit(@OnRunWithoutDebugInit);
  MultiProjectManager.AddHandlerOnRunTyphonbuild(@OnMultiProjectRunTyphonbuild);  //=== ct9999 =========
end;

procedure TPJSController.UnHook;
begin
  FMacroPas2js:=nil;
end;

procedure TPJSController.StoreMacros;
begin
  if PJSOptions=nil then exit;
  FMacroPas2js.TyphonbuildValue:=PJSOptions.GetParsedCompilerFilename;
end;

//=== ct9999 ==============================================

function TPJSController.GetPas2JSWebServerPath(const s: string;
  const Data: PtrInt; var Abort: boolean): string;
begin
  Abort:=False;
  if (s<>'') and (ConsoleVerbosity>=0) then
    debugln(['Hint: (IDE) [TPJSController.GetPas2JSWebServerPath] ignoring macro Pas2JSWebServer parameter "',s,'"']);
  Result:=PJSOptions.GetParsedWebServerFilename;

  if FileExists(Result)=False then
     Result:=ctUtils.ctGetFPCEXEDir(True)+PJSDefaultWebServerName+GetExeExt;
end;

function TPJSController.GetPas2JSWebServerPort(const s: string;
  const Data: PtrInt; var Abort: boolean): string;
begin
  Abort:=False;
  if (s<>'') and (ConsoleVerbosity>=0) then
    debugln(['Hint: (IDE) [TPJSController.GetPas2JSWebServerPort] ignoring macro Pas2JSWebServerPort parameter "',s,'"']);
  Result:=PJSOptions.GetParsedWebServerFilename;
  if Result='' then
    Result:=PJSDefaultWebServerName; // always return something to get nicer error messages
end;

procedure TPJSController.RefreshView;
begin
  If Assigned(FOnRefresh) then
    FOnRefresh(Self);
end;

{ TServerInstanceList }

function TServerInstanceList.GetInstance(AIndex : Integer): TServerInstance;
begin
  Result:=Items[AIndex] as TServerInstance;
end;

function TServerInstanceList.IndexOfPort(APort: Word): integer;
begin
  Result:=Count-1;
  While (Result>=0) and (GetInstance(Result).Port<>APort) do Dec(Result);
end;

function TServerInstanceList.FindByPort(Aindex: Integer): TServerInstance;

Var
  I : Integer;

begin
  I:=IndexOfPort(Aindex);
  If I=-1 then
    Result:=nil
  else
    Result:=GetInstance(I);
end;

function TServerInstanceList.AddInstance(aPort: Word; const ABaseURL,
  aServerName: String): TServerInstance;
begin
  Result:=Add as TServerInstance;
  Result.Port:=aPort;
  Result.BaseDir:=ABaseURL;
  Result.ServerName:=aServerName;
end;

{ TServerInstance }

function TServerInstance.GetRunning: Boolean;
begin
  Result:=Assigned(FProcess);
  if Result then
    Result:=Process.Running;
end;

destructor TServerInstance.Destroy;
begin
  StopServer;
  FreeAndNil(FProcess);
  inherited;
end;

procedure TServerInstance.StartServer;
begin
  if Running then
    exit;
  If not Assigned(FProcess) then
    FProcess:=TProcess.Create(Nil);
  FProcess.Executable:=ServerName;
  FProcess.Parameters.Add('-q');
  FProcess.Parameters.Add('-p');
  FProcess.Parameters.Add(IntToStr(Port));
  FProcess.Parameters.AddStrings(PJSOptions.HTTPServerOpts);
  {$IFDEF WINDOWS}
  FProcess.Options:=[poNoConsole];
  {$ENDIF}
  if ConsoleVerbosity>=0 then
    DebugLN(['Starting server from Directory : ',BaseDir]);
  FProcess.CurrentDirectory:=BaseDir;
  try
    FProcess.Execute;
  except
    On E : Exception do
      begin
      FRunError:=E.Message;
      Raise;
      end;
  end;
  TPJSController.Instance.RefreshView;
end;

procedure TServerInstance.StopServer;
begin
  if Running then
    FProcess.Terminate(0);
  TPJSController.Instance.RefreshView;
end;

//=========================================================

finalization
  TPJSController.DoneInstance;
end.


