{ pas2js options

  Author: Mattias Gaertner
}
unit PJSDsgnOptions;

{$mode objfpc}{$H+}
{$Inline on}

interface

uses
  Classes, SysUtils,
  // BaseUtils
  LazFileCache, LazConfigStorage, LazFileUtils, FileUtil,
  // Codetools
  DefineTemplates, FileProcs,
  // IdeIntf
  MacroIntf, BaseIDEIntf;

const
  PJSDsgnOptsFile = 'pas2jsdsgnoptions.xml';
  //=== ct9999 =====================================
  PJSDefaultCompiler = '$MakeExe(IDE,$(FPCEXEDir)/pas2js)';
  PJSDefaultPas2jsSrcDir = '';
  PJSDefaultFPCSrcDir = '$(Pas2jsSrcDir)/compiler';
  PJSDefaultFPCExe= '';
  PJSDefaultDTS2Pas  = '$MakeExe(IDE,$(FPCEXEDir)/dts2pas)';
  PJSDefaultWebServerName = 'compileserver';
  PJSDefaultWebServer = '$MakeExe(IDE,$(FPCEXEDir)/'+PJSDefaultWebServerName+')';
  //================================================
  PJSDefaultDTS2PasService = 'https://www.freepascal.org/~michael/service/dts2pas.cgi';
  PJSDefaultStartAtPort = 3000; // compileserver default port
  PJSDefaultHTTPServerParams = '-s -p $()';
  PJSDefaultBrowser = '$MakeExe(IDE,firefox)';
  PJSDefaultNodeJS = '$MakeExe(IDE,nodejs)';
  PJSDefaultElectronExe = '$MakeExe(IDE,electron)';

Type
  TPas2jsCachedOption = (
    p2jcoCompilerFilename,
    p2jcoPas2jsSrcDir,
    p2jcoFPCSrcDir, // fpc source directory used for compiling tools and pas2js
    p2jcoFPCExe, // fpc used for compiling tools
    //=== ct9999 =================
    p2jcoBrowserFilename,
    p2jcoHTTPServerFilename,
    p2jcoHTTPServerPort,
    //============================
    p2jcoNodeJSFilename,
    p2jcoElectronFilename,
    p2jcoAtomTemplateDir,
    p2jcoVSCodeTemplateDir,
    p2jcoDTSToPas,
    p2jcoDTSToPasServiceURL
    );
  TPas2jsCachedOptions = set of TPas2jsCachedOption;

const
  p2jcoFilenames = [
    p2jcoCompilerFilename,
    p2jcoFPCExe,
    //=== ct9999 =================
    p2jcoBrowserFilename,
    p2jcoHTTPServerFilename,
    //============================
    p2jcoNodeJSFilename,
    p2jcoElectronFilename,
    p2jcoAtomTemplateDir,
    p2jcoVSCodeTemplateDir,
    p2jcoDTSToPas
  ];
  p2jcoDirectories = [
    p2jcoPas2jsSrcDir,
    p2jcoFPCSrcDir
  ];

type
  TPas2jsCachedValue = record
    RawValue: string;
    Stamp: int64;
    ParsedValue: string;
  end;
  PPas2jsCachedValue = ^TPas2jsCachedValue;

  { TPas2jsOptions }

  TPas2jsOptions = class
  private
    FBrowserFileName: String;
    FCachedOptions: array[TPas2jsCachedOption] of TPas2jsCachedValue;
    FChangeStamp: int64;
    FOldWebServerFileName: string;
    FSavedStamp: int64;
    FStartAtPort: Word;
    //=== ct9999 ============================
    FHTTPServerOpts: TStrings;
    procedure DoOptsChanged(Sender: TObject);
    function GetStartAtPort: Word;
    function GetWebServerFileName: string;
    procedure SetWebServerFileName(AValue: string);
    procedure SetHTTPServerOpts(AValue: TStrings);
    function GetBrowserFileName: String;
    procedure SetBrowserFileName(AValue: String);
    //======================================
    function GetAtomTemplateDir: String;
    function GetCompilerFilename: string;
    function GetDTS2Pas: String;
    function GetDTS2PasService: String;
    function GetElectronFileName: string;
    function GetFPCExe: string;
    function GetFPCSrcDir: string;
    function GetPas2jsSrcDir: string;
    function GetVSCodeTemplateDir: String;
    function GetModified: boolean;
    function GetNodeJSFileName: string;
    function GetParsedOptionValue(Option: TPas2jsCachedOption): string;
    procedure SetAtomTemplateDir(AValue: String);
    procedure SetDTS2Pas(AValue: String);
    procedure SetDTS2PasService(AValue: String);
    procedure SetElectronFileName(AValue: string);
    procedure SetFPCExe(AValue: string);
    procedure SetFPCSrcDir(AValue: string);
    procedure SetPas2jsSrcDir(AValue: string);
    procedure SetVSCodeTemplateDir(AValue: String);
    procedure SetModified(AValue: boolean);
    procedure SetCompilerFilename(AValue: string);
    procedure SetNodeJSFileName(AValue: string);
    procedure SetStartAtPort(AValue: Word);
    procedure SetCachedOption(Option: TPas2jsCachedOption; const AValue: string);
  public
    constructor Create;
    destructor Destroy; override;
    procedure IncreaseChangeStamp; inline;
    procedure Load;
    procedure Save;
    function GetParsedCompilerFilename: string;
    function GetParsedPas2jsSrcDir: string;
    function GetParsedNodeJSFilename: string;
    function GetParsedElectronExe: string;
    procedure LoadFromConfig(Cfg: TConfigStorage);
    procedure SaveToConfig(Cfg: TConfigStorage);
  public
    property ChangeStamp: int64 read FChangeStamp;
    property Modified: boolean read GetModified write SetModified;
    property CompilerFilename: string read GetCompilerFilename write SetCompilerFilename;
    property Pas2jsSrcDir: string read GetPas2jsSrcDir write SetPas2jsSrcDir; // with trailing pathdelim
    property FPCSrcDir: string read GetFPCSrcDir write SetFPCSrcDir; // used for compiling tools and pas2js, with trailing pathdelim
    property FPCExe: string read GetFPCExe write SetFPCExe; // used for compiling tools
    property NodeJSFileName : string Read GetNodeJSFileName Write SetNodeJSFileName;
    property ElectronFileName : string Read GetElectronFileName Write SetElectronFileName;
    property AtomTemplateDir : String Read GetAtomTemplateDir Write SetAtomTemplateDir;
    property VSCodeTemplateDir : String Read GetVSCodeTemplateDir Write SetVSCodeTemplateDir;
    property DTS2Pas : String Read GetDTS2Pas Write SetDTS2Pas;
    property DTS2PasServiceURL : String Read GetDTS2PasService Write SetDTS2PasService;
    property StartAtPort : Word Read FStartAtPort Write SetStartAtPort;
  public
    property OldWebServerFileName: string read FOldWebServerFileName write FOldWebServerFileName;
    //=== ct9999 ==============================             
    function GetParsedBrowserFilename: string;                           
    function GetParsedWebServerFilename: string;
    function GetParsedWebServerPort: string;
    Property BrowserFileName : String Read GetBrowserFileName Write SetBrowserFileName;
    Property WebServerFileName : string Read GetWebServerFileName Write SetWebServerFileName;
    Property HTTPServerOpts : TStrings Read FHTTPServerOpts Write SetHTTPServerOpts;
    //========================================
  end;

var
  PJSOptions: TPas2jsOptions = nil;

function GetStandardPas2jsExe: string;
function GetStandardElectron: string;
function GetStandardNodeJS: string;
function GetPas2jsQuality(Filename: string; out Msg: string): boolean;

//=== ct9999 =====================================  
function GetStandardBrowser: string;
function GetStandardWebServerExe: string;

implementation

uses
  strpas2jsdesign;

function GetStandardWebServerExe: string;
begin
  Result:=PJSDefaultWebServer;
  if not IDEMacros.SubstituteMacros(Result) then
    Result:='PJSDefaultHTTPServerCmd'+GetExeExt;
end;

function GetStandardBrowser: string;
begin
  Result:='$MakeExe(IDE,firefox)';
  if not IDEMacros.SubstituteMacros(Result) then
    begin
    Result:='$MakeExe(IDE,chrome)';
    {$ifdef windows}
    if not IDEMacros.SubstituteMacros(Result) then
      Result:='$MakeExe(IDE,iexplore)';
    {$else}
    {$ifdef darwin}
     if not IDEMacros.SubstituteMacros(Result) then
       Result:='$MakeExe(IDE,xdg-open)';
    {$endif}
     if not IDEMacros.SubstituteMacros(Result) then
       Result:='';
    {$endif}
    end;
end;
//===================================================

function GetStandardPas2jsExe: string;
begin
  Result:=PJSDefaultCompiler;
  if not IDEMacros.SubstituteMacros(Result) then
    Result:='pas2js'+GetExeExt;
end;

function GetStandardNodeJS: string;
begin
  Result:=PJSDefaultNodeJS;
  if not IDEMacros.SubstituteMacros(Result) then
    Result:='nodejs'+GetExeExt;
end;

function GetStandardElectron: string;
begin
  Result:='electron'+GetExeExt;
end;

function GetPas2jsQuality(Filename: string; out Msg: string): boolean;
var
  ShortFile: String;
begin
  Msg:='';
  Filename:=TrimFilename(Filename);
  if (Filename='') then begin
    Msg:=pjsdMissingPathToPas2js;
    exit(false);
  end;
  if not FileExistsCached(Filename) then begin
    Msg:=CTSafeFormat(pjsdFileNotFound, [Filename]);
    exit(false);
  end;
  if not DirPathExistsCached(ExtractFilePath(Filename)) then begin
    Msg:=CTSafeFormat(pjsdDirectoryNotFound, [ExtractFilePath(Filename)]);
    exit(false);
  end;
  if not FileIsExecutable(Filename) then begin
    Msg:=CTSafeFormat(pjsdFileNotExecutable, [Filename]);
    exit(false);
  end;
  ShortFile:=ExtractFileNameOnly(Filename);
  if not CompareText(LeftStr(ShortFile,length('pas2js')),'pas2js')<>0 then begin
    Msg:=pjsdFileNameDoesNotStartWithPas2js;
    exit(false);
  end;
  // run it
  //RunTool(Filename);
end;

{ TPas2jsOptions }

procedure TPas2jsOptions.SetModified(AValue: boolean);
begin
  if AValue then
    IncreaseChangeStamp
  else
    FSavedStamp:=FChangeStamp;
end;

function TPas2jsOptions.GetModified: boolean;
begin
  Result:=FSavedStamp<>FChangeStamp;
end;

function TPas2jsOptions.GetAtomTemplateDir: String;
begin
  Result:=FCachedOptions[p2jcoAtomTemplateDir].RawValue;
end;

function TPas2jsOptions.GetCompilerFilename: string;
begin
  Result:=FCachedOptions[p2jcoCompilerFilename].RawValue;
end;

function TPas2jsOptions.GetDTS2Pas: String;
begin
  Result:=FCachedOptions[p2jcoDTSToPas].RawValue
end;

function TPas2jsOptions.GetDTS2PasService: String;
begin
  Result:=FCachedOptions[p2jcoDTSToPasServiceURL].RawValue
end;

function TPas2jsOptions.GetElectronFileName: string;
begin
  Result:=FCachedOptions[p2jcoElectronFilename].RawValue;
end;

function TPas2jsOptions.GetFPCExe: string;
begin
  Result:=FCachedOptions[p2jcoFPCExe].RawValue;
end;

function TPas2jsOptions.GetFPCSrcDir: string;
begin
  Result:=FCachedOptions[p2jcoFPCSrcDir].RawValue;
end;

function TPas2jsOptions.GetPas2jsSrcDir: string;
begin
  Result:=FCachedOptions[p2jcoPas2jsSrcDir].RawValue;
end;

function TPas2jsOptions.GetVSCodeTemplateDir: String;
begin
  Result:=FCachedOptions[p2jcoVSCodeTemplateDir].RawValue;
end;

function TPas2jsOptions.GetNodeJSFileName: string;
begin
  Result:=FCachedOptions[p2jcoNodeJSFilename].RawValue;
end;

procedure TPas2jsOptions.SetCompilerFilename(AValue: string);
begin
  AValue:=TrimFilename(AValue);
  SetCachedOption(p2jcoCompilerFilename,AValue);
end;

procedure TPas2jsOptions.SetNodeJSFileName(AValue: string);
begin
  AValue:=TrimFilename(AValue);
  SetCachedOption(p2jcoNodeJSFilename,AValue);
end;

constructor TPas2jsOptions.Create;
var
  o: TPas2jsCachedOption;
begin
  FChangeStamp:=LUInvalidChangeStamp64;
  FCachedOptions[p2jcoCompilerFilename].RawValue:=PJSDefaultCompiler;
  //=== ct9999 =================== 
  FCachedOptions[p2jcoHTTPServerFilename].RawValue:=PJSDefaultWebServer;  
  FCachedOptions[p2jcoBrowserFilename].RawValue:=PJSDefaultBrowser;
  //==============================  
  FCachedOptions[p2jcoNodeJSFilename].RawValue:=PJSDefaultNodeJS;
  FCachedOptions[p2jcoElectronFilename].RawValue:=PJSDefaultElectronExe;
  FCachedOptions[p2jcoAtomTemplateDir].RawValue:='';
  FCachedOptions[p2jcoVSCodeTemplateDir].RawValue:='';
  FCachedOptions[p2jcoDTSToPas].RawValue:=PJSDefaultDTS2Pas;
  FCachedOptions[p2jcoDTSToPasServiceURL].RawValue:=PJSDefaultDTS2PasService;
  for o in TPas2jsCachedOption do
    FCachedOptions[o].Stamp:=LUInvalidChangeStamp64;
  //=== ct9999 ===================
  FHTTPServerOpts:=TStringList.Create;
  TStringList(FHTTPServerOpts).OnChange:=@DoOptsChanged;
  //==============================
end;

destructor TPas2jsOptions.Destroy;
begin
  FreeAndNil(FHTTPServerOpts);  //=== ct9999 =======
  inherited Destroy;
end;

procedure TPas2jsOptions.IncreaseChangeStamp;
begin
  LUIncreaseChangeStamp64(FChangeStamp);
end;

procedure TPas2jsOptions.Load;
var
  Cfg: TConfigStorage;
begin
  Cfg:=GetIDEConfigStorage(PJSDsgnOptsFile,true);
  try
    LoadFromConfig(Cfg);
  finally
    Cfg.Free;
  end;
end;

procedure TPas2jsOptions.Save;
var
  Cfg: TConfigStorage;
begin
  Cfg:=GetIDEConfigStorage(PJSDsgnOptsFile,false);
  try
    SaveToConfig(Cfg);
  finally
    Cfg.Free;
  end;
end;

Const
  KeyCompiler = 'compiler/value';
  KeyPas2jsSrcDir = 'sources/value';
  KeyFPCSrcDir = 'fpcsrcdir/value';
  KeyFPCExe= 'fpc/value';
  KeyHTTPServer = 'webserver/value';
  KeyHTTPServerOptions = 'webserver/extraoptions/value'; //=== ct9999 =======
  KeyBrowser = 'webbrowser/value';
  KeyNodeJS = 'nodejs/value';
  KeyElectronExe = 'electron/value';
  KeyAtomTemplate = 'atomtemplate/value';
  KeyVSCodeTemplate = 'vscodetemplate/value';
  KeyStartPortAt = 'webserver/startatport/value';
  KeyDTS2PasTool = 'dts2pastool/value';
  KeyDTS2PasServiceURL= 'dts2passerviceurl/value';

procedure TPas2jsOptions.LoadFromConfig(Cfg: TConfigStorage);

begin
  CompilerFilename:=Cfg.GetValue(KeyCompiler,PJSDefaultCompiler);
  Pas2jsSrcDir:=Cfg.GetValue(KeyPas2jsSrcDir,PJSDefaultPas2jsSrcDir);
  FPCSrcDir:=Cfg.GetValue(KeyFPCSrcDir,PJSDefaultFPCSrcDir);
  FPCExe:=Cfg.GetValue(KeyFPCExe,PJSDefaultFPCExe);
  //=== ct9999 ======================================
  WebServerFileName:=Cfg.GetValue(KeyHTTPServer,PJSDefaultWebServer); 
  BrowserFileName:=Cfg.GetValue(KeyBrowser,PJSDefaultBrowser);
  //=================================================
  NodeJSFileName:=Cfg.GetValue(KeyNodeJS,PJSDefaultNodeJS);
  ElectronFileName:=Cfg.GetValue(KeyElectronExe,PJSDefaultElectronExe);
  AtomTemplateDir:=Cfg.GetValue(KeyAtomTemplate,'');
  VSCodeTemplateDir:=Cfg.GetValue(KeyVSCodeTemplate,'');
  DTS2Pas:=cfg.GetValue(KeyDTS2PasTool,PJSDefaultDTS2Pas);
  DTS2PasServiceURL:=cfg.GetValue(KeyDTS2PasServiceURL,PJSDefaultDTS2PasService);
  StartAtPort :=Cfg.GetValue(KeyStartPortAt,PJSDefaultStartAtPort);
  
  //=== ct9999 =================
  Cfg.GetValue(KeyHTTPServerOptions,FHTTPServerOpts);
  //============================
  Modified:=false;
end;

procedure TPas2jsOptions.SaveToConfig(Cfg: TConfigStorage);

begin
  Cfg.SetDeleteValue(KeyCompiler,CompilerFilename,PJSDefaultCompiler);
  Cfg.SetDeleteValue(KeyPas2jsSrcDir,Pas2jsSrcDir,PJSDefaultPas2jsSrcDir);
  Cfg.SetDeleteValue(KeyFPCSrcDir,FPCSrcDir,PJSDefaultFPCSrcDir);
  Cfg.SetDeleteValue(KeyFPCExe,FPCExe,PJSDefaultFPCExe);
  //=== ct9999 ======================================
  Cfg.SetDeleteValue(KeyHTTPServer,WebServerFileName,PJSDefaultWebServer);
  Cfg.SetDeleteValue(KeyBrowser,BrowserFileName,PJSDefaultBrowser);
  //=================================================
  Cfg.SetDeleteValue(KeyStartPortAt,StartAtPort,PJSDefaultStartAtPort);
  Cfg.SetDeleteValue(KeyNodeJS,NodeJSFileName,PJSDefaultNodeJS);
  Cfg.SetDeleteValue(KeyElectronExe,ElectronFileName,PJSDefaultElectronExe);
  Cfg.SetDeleteValue(KeyAtomTemplate,AtomTemplateDir,'');
  Cfg.SetDeleteValue(KeyVSCodeTemplate,VSCodeTemplateDir,'');
  cfg.SetDeleteValue(KeyDTS2PasTool,DTS2Pas,PJSDefaultDTS2Pas);
  cfg.SetDeleteValue(KeyDTS2PasServiceURL,DTS2PasServiceURL,PJSDefaultDTS2PasService);
  
  //=== ct9999 =================
  Cfg.SetValue(KeyHTTPServerOptions,FHTTPServerOpts);
  //============================
  Modified:=false;
end;

function TPas2jsOptions.GetParsedCompilerFilename: string;
begin
  Result:=GetParsedOptionValue(p2jcoCompilerFilename);
end;

function TPas2jsOptions.GetParsedPas2jsSrcDir: string;
begin
  Result:=GetParsedOptionValue(p2jcoPas2jsSrcDir);
end;

function TPas2jsOptions.GetParsedNodeJSFilename: string;
begin
  Result:=GetParsedOptionValue(p2jcoNodeJSFilename);
end;

function TPas2jsOptions.GetParsedElectronExe: string;
begin
  Result:=GetParsedOptionValue(p2jcoElectronFilename);
end;

function TPas2jsOptions.GetParsedOptionValue(Option: TPas2jsCachedOption
  ): string;
var
  p: PPas2jsCachedValue;
  IsFilename, IsDirectory: Boolean;
begin
  p:=@FCachedOptions[Option];
  if p^.Stamp<>IDEMacros.BaseTimeStamp then
  begin
    p^.Stamp:=IDEMacros.BaseTimeStamp;
    p^.ParsedValue:=p^.RawValue;
    IDEMacros.SubstituteMacros(p^.ParsedValue);
    IsFilename:=Option in p2jcoFilenames;
    IsDirectory:=Option in p2jcoDirectories;
    if IsFilename then
    begin
      p^.ParsedValue:=TrimFilename(p^.ParsedValue);
      if (p^.ParsedValue<>'')
          and not FilenameIsAbsolute(p^.ParsedValue) then
      begin
        if ExtractFilePath(p^.ParsedValue)='' then
          p^.ParsedValue:=FindDefaultExecutablePath(p^.ParsedValue)
        else
          p^.ParsedValue:=''; // not found
      end;
    end;
    if IsDirectory then
    begin
      p^.ParsedValue:=TrimFilename(p^.ParsedValue);
      if (p^.ParsedValue<>'')
          and not FilenameIsAbsolute(p^.ParsedValue) then
      begin
        p^.ParsedValue:=''; // not found
      end;
    end;
    if p^.ParsedValue='' then
    begin
      case Option of
        p2jcoCompilerFilename: p^.ParsedValue:=GetStandardPas2jsExe;
        //=== ct9999 ============================
        p2jcoBrowserFilename: p^.ParsedValue:=GetStandardBrowser;
        p2jcoHTTPServerFilename: p^.ParsedValue:=GetStandardWebServerExe;
        p2jcoHTTPServerPort: p^.ParsedValue:=IntToStr(PJSDefaultStartAtPort);
        //=======================================
        p2jcoNodeJSFilename: p^.ParsedValue:=GetStandardNodeJS;
        p2jcoElectronFilename: p^.ParsedValue:=GetStandardElectron;
      end;
      if IsFilename and (p^.ParsedValue<>'')
          and not FilenameIsAbsolute(p^.ParsedValue) then
      begin
        if ExtractFilePath(p^.ParsedValue)='' then
          p^.ParsedValue:=FindDefaultExecutablePath(p^.ParsedValue)
        else
          p^.ParsedValue:=''; // not found
      end;
    end;
  end;
  Result:=p^.ParsedValue;
end;

procedure TPas2jsOptions.SetAtomTemplateDir(AValue: String);
begin
  AValue:=TrimFilename(AValue);
  SetCachedOption(p2jcoAtomTemplateDir,AValue);
end;

procedure TPas2jsOptions.SetDTS2Pas(AValue: String);
begin
  AValue:=TrimFilename(AValue);
  SetCachedOption(p2jcoDTSToPas,AValue);
end;

procedure TPas2jsOptions.SetDTS2PasService(AValue: String);
begin
  SetCachedOption(p2jcoDTSToPasServiceURL,AValue);
end;

procedure TPas2jsOptions.SetElectronFileName(AValue: string);
begin
  AValue:=TrimFilename(AValue);
  SetCachedOption(p2jcoElectronFilename,AValue);
end;

procedure TPas2jsOptions.SetFPCExe(AValue: string);
begin
  AValue:=TrimFilename(AValue);
  SetCachedOption(p2jcoFPCExe,AValue);
end;

procedure TPas2jsOptions.SetFPCSrcDir(AValue: string);
begin
  AValue:=AppendPathDelim(TrimFilename(AValue));
  SetCachedOption(p2jcoFPCSrcDir,AValue);
end;

procedure TPas2jsOptions.SetPas2jsSrcDir(AValue: string);
begin
  AValue:=AppendPathDelim(TrimFilename(AValue));
  SetCachedOption(p2jcoPas2jsSrcDir,AValue);
end;

procedure TPas2jsOptions.SetVSCodeTemplateDir(AValue: String);
begin
  AValue:=TrimFilename(AValue);
  SetCachedOption(p2jcoVSCodeTemplateDir,AValue);
end;

procedure TPas2jsOptions.SetStartAtPort(AValue: Word);
begin
  if FStartAtPort=AValue then Exit;
  FStartAtPort:=AValue;
  IncreaseChangeStamp;
end;

procedure TPas2jsOptions.SetCachedOption(Option: TPas2jsCachedOption;
  const AValue: string);
begin
  if FCachedOptions[Option].RawValue=AValue then exit;
  FCachedOptions[Option].RawValue:=AValue;
  FCachedOptions[Option].Stamp:=InvalidIDEMacroStamp;
  IncreaseChangeStamp;
  IDEMacros.IncreaseBaseStamp;
end;

Procedure DonePSJOptions;

begin
  if PJSOptions<>nil then
  begin
    try
      if PJSOptions.Modified then
        PJSOptions.Save;
    except
    end;
    FreeAndNil(PJSOptions);
  end;
end;

//=== ct9999 ================================================

function TPas2jsOptions.GetBrowserFileName: String;
begin
  Result:=FCachedOptions[p2jcoBrowserFilename].RawValue;
end;

procedure TPas2jsOptions.SetBrowserFileName(AValue: String);
begin
  AValue:=TrimFilename(AValue);
  SetCachedOption(p2jcoBrowserFilename,AValue);
end;        

function TPas2jsOptions.GetParsedBrowserFilename: string;
begin
  Result:=GetParsedOptionValue(p2jcoBrowserFilename);
end;

function TPas2jsOptions.GetParsedWebServerFilename: string;
begin
  Result:=GetParsedOptionValue(p2jcoHTTPServerFilename);
end;

function TPas2jsOptions.GetParsedWebServerPort: string;
begin
  Result:=GetParsedOptionValue(p2jcoHTTPServerPort);
end;

procedure TPas2jsOptions.DoOptsChanged(Sender: TObject);
begin
  Self.Modified:=True;
end;

function TPas2jsOptions.GetStartAtPort: Word;
begin
  Result:=StrToIntDef(FCachedOptions[p2jcoHTTPServerPort].RawValue,PJSDefaultStartAtPort);
end;

function TPas2jsOptions.GetWebServerFileName: string;
begin
  Result:=FCachedOptions[p2jcoHTTPServerFilename].RawValue;
end;

procedure TPas2jsOptions.SetWebServerFileName(AValue: string);
begin
  AValue:=TrimFilename(AValue);
  SetCachedOption(p2jcoHTTPServerFilename,AValue);
end;

procedure TPas2jsOptions.SetHTTPServerOpts(AValue: TStrings);
begin
  if FHTTPServerOpts=AValue then Exit;
  FHTTPServerOpts.Assign(AValue);
end;

//=============================================================

finalization
  DonePSJOptions;
end.
